#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2014 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Encinitas reports
"""

from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zc.table.column import GetterColumn
from reportlab.lib import units

from schooltool import table
from schooltool.basicperson.interfaces import IDemographics
from schooltool.common import SchoolToolMessage as STMessage
from schooltool.course.browser.section import SectionPDFViewBase
from schooltool.group.browser.group import number_getter
from schooltool.person.interfaces import IPersonFactory
from schooltool.report.browser.report import RequestRemoteReportDialog
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.skin import flourish

from schooltool.encinitas import EncinitasMessage as _


class RequestSectionReportView(RequestRemoteReportDialog):

    pass


class ReportViewBase(flourish.report.PlainPDFPage):

    pass


class AlertsListPDFView(ReportViewBase):

    name = _('Alerts List')

    @property
    def message_title(self):
        return _("group ${title} alerts list",
                 mapping={'title': self.context.title})

    @property
    def scope(self):
        schoolyear = ISchoolYear(self.context.__parent__)
        return schoolyear.title

    @property
    def title(self):
        return self.context.title

    @property
    def base_filename(self):
        return 'group_alerts_list_%s' % self.context.__name__


class SectionAlertsListPDFView(SectionPDFViewBase, AlertsListPDFView):

    @property
    def message_title(self):
        return _("section ${title} alerts list",
                 mapping={'title': self.context.title})
    @property
    def base_filename(self):
        courses = [c.__name__ for c in self.context.courses]
        return 'section_alerts_list_%s' % '_'.join(courses)


class ProgressReportPDFView(SectionPDFViewBase, ReportViewBase):

    name = _('Report to the Director')

    @property
    def message_title(self):
        return _("section ${title} progress report",
                 mapping={'title': self.context.title})
    @property
    def base_filename(self):
        courses = [c.__name__ for c in self.context.courses]
        return 'section_progress_report_%s' % '_'.join(courses)


class AlertsListTablePart(table.pdf.RMLTablePart):

    table_name = 'alerts_list_table'

    def getColumnWidths(self, rml_columns):
        return '5% 25% 15% 55%'


class AlertsListTable(table.ajax.Table):

    batch_size = 0
    visible_column_names = ['number', 'name', 'birth_date', 'alert']

    def items(self):
        return self.context.members

    def sortOn(self):
        return getUtility(IPersonFactory).sortOn()

    def columns(self):
        first_name = table.column.LocaleAwareGetterColumn(
            name='first_name',
            title=u'First Name',
            getter=lambda i, f: i.first_name,
            subsort=True)
        last_name = table.column.LocaleAwareGetterColumn(
            name='last_name',
            title=u'Last Name',
            getter=lambda i, f: i.last_name,
            subsort=True)
        number = GetterColumn(
            name='number',
            title=u'#',
            getter=number_getter)
        name = table.column.LocaleAwareGetterColumn(
            name='name',
            title=STMessage(u'Name'),
            getter=lambda i, f: i.title,
            subsort=True)
        birth_date = GetterColumn(
            name='birth_date',
            title=STMessage(u'Birth date'),
            getter=lambda i, f: i.birth_date)
        alert = GetterColumn(
            name='alert',
            title=_(u'Alert Comment'),
            getter=lambda i, f: IDemographics(i).get('alert_comment'))
        return [first_name, last_name, number, name, birth_date, alert]


class RMLAlertColumn(table.pdf.RMLGetterColumn):

    def renderCell(self, item, formatter):
        value = self.column.getter(item, formatter)
        if value is not None:
            html2rml = getMultiAdapter((unicode(value), self.request),
                                       name='html2rml')
            html2rml.para_class = 'table-cell'
            return html2rml


class ProgressReportTablePart(table.pdf.RMLTablePart):

    table_name = 'progress_report_table'
    table_style = 'progress_report'
    template = flourish.templates.XMLFile('rml/progress_report.pt')

    def getColumnWidths(self, rml_columns):
        return '5% 26.2% 7.6% 7.6% 7.6% 11.5% 11.5% 11.5% 11.5%'

    def getHeaders(self, rml_columns):
        rows= [
            [
                '#',
                STMessage('Name'),
                _('Academic Progress'),
                '',
                '',
                _('Social Progress'),
                '',
                _('Parents'),
                '',
            ],
            [
                '',
                '',
                STMessage('Grade Level'),
                '',
                '',
                _('Doing fine'),
                _('Needs attention *'),
                _('Satisfied'),
                _('Needs attention *'),
            ],
            [
                '',
                '',
                _('at'),
                _('above'),
                _('below'),
                '',
                '',
                '',
                '',
            ]
        ]
        result = []
        for i, row in enumerate(rows):
            result.append([])
            current = result[i]
            for column in row:
                current.append(translate(column, context=self.request))
        return result


class DummyColumn(GetterColumn):

    def __init__(self, name):
        super(DummyColumn, self).__init__(name=name, title=u'',
                                          getter=lambda i, f: None)


class ProgressReportTable(AlertsListTable):

    visible_column_names = ['number', 'name',
                            'academic_1', 'academic_2', 'academic_3',
                            'social_1', 'social_2',
                            'parents_1', 'parents_2']

    def columns(self):
        default_columns = super(ProgressReportTable, self).columns()
        first_name = default_columns[0]
        last_name = default_columns[1]
        number = default_columns[2]
        name = default_columns[3]
        academic_1 = DummyColumn('academic_1')
        academic_2 = DummyColumn('academic_2')
        academic_3 = DummyColumn('academic_3')
        social_1 = DummyColumn('social_1')
        social_2 = DummyColumn('social_2')
        parents_1 = DummyColumn('parents_1')
        parents_2 = DummyColumn('parents_2')
        return [first_name, last_name, number, name,
                academic_1, academic_2, academic_3,
                social_1, social_2,
                parents_1, parents_2]


class ProgressReportPageTemplate(flourish.report.PlainPageTemplate):

    @Lazy
    def frame(self):
        doc_w, doc_h = self.manager.page_size
        margin = flourish.report.Box(8, 0, 2*units.inch, 0)
        width = (doc_w - self.manager.margin.left - self.manager.margin.right
                 - margin.left - margin.right)
        height = (doc_h - self.manager.margin.top - self.manager.margin.bottom
                  - self.top_bar['height'] - self.bottom_bar['height']
                  - self.header['height']
                  - margin.top - margin.bottom)
        x = self.manager.margin.left + margin.left
        y = (self.manager.margin.bottom + self.bottom_bar['height'] +
             margin.bottom)
        return {
            'height': height,
            'margin': margin,
            'width': width,
            'x': x,
            'y': y,
            }

    @Lazy
    def clarify_area(self):
        return {
            'width': self.frame['width'],
            'height': self.frame['margin'].bottom,
            'x': self.frame['x'],
            'y': self.bottom_bar['y'] + self.bottom_bar['height'],
        }

    @Lazy
    def clarify_lines(self):
        result = []
        interline_space = 16
        count = 7
        y = (self.clarify_area['y'] + self.clarify_area['height'] -
             (interline_space * 2))
        x1 = self.clarify_area['x']
        x2 = self.clarify_area['x'] + self.clarify_area['width']
        for i in range(count):
            result.append(' '.join(['%.2f' % j for j in [x1, y, x2, y]]))
            y -= interline_space
        return ' '.join(result)
